﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TestProject
{
    public class DataGridViewRichTextBoxColumn : DataGridViewTextBoxColumn
    {
        public DataGridViewRichTextBoxColumn()
        {
            CellTemplate = new DataGridViewRichTextBoxCell();
        }
    }

    public class DataGridViewRichTextBoxCell: DataGridViewTextBoxCell
    {
        static DOMParser parser = new DOMParser();
        public DataGridViewRichTextBoxCell()
            :base()
        {
        }



        public DOMNode Document { get; set; }

        protected override void Paint(System.Drawing.Graphics g, System.Drawing.Rectangle clipBounds, System.Drawing.Rectangle cellBounds, int rowIndex, DataGridViewElementStates cellState, object value, object formattedValue, string errorText, DataGridViewCellStyle cellStyle, DataGridViewAdvancedBorderStyle advancedBorderStyle, DataGridViewPaintParts paintParts)
        {
            //背景色
            bool selected = (cellState & DataGridViewElementStates.Selected) == DataGridViewElementStates.Selected;
            Color clr_background = !selected
                                       ? cellStyle.BackColor
                                       : cellStyle.SelectionBackColor;
            using (Brush bru = new SolidBrush(clr_background))
            {
                g.FillRectangle(bru, cellBounds);
            }
            
            //边框
            if ((paintParts & DataGridViewPaintParts.Border) != 0)
                PaintBorder(g, clipBounds, cellBounds, cellStyle, advancedBorderStyle);

            Rectangle rect_border = BorderWidths(advancedBorderStyle);
            Rectangle rect = cellBounds;
            rect.Offset(rect_border.X, rect_border.Y);
            rect.Width -= rect_border.Right;
            rect.Height -= rect_border.Bottom;

            if (value != null)
            {
                string text = value.ToString();
                if (text.Length > 0)
                    PaintText(g, rect, text, selected, cellStyle);
            }
        }

        void PaintText(Graphics g, Rectangle rect, string text, bool selected, DataGridViewCellStyle cell_style)
        {
            Document = parser.parse_text(text);
            Point pt = rect.Location;
            foreach (var node in Document.Nodes)
            {
                Font font = get_node_font(node, cell_style);
                Color clr = get_node_color(node, selected, cell_style);
                Size size = TextRenderer.MeasureText(node.InnerText, font);
                pt.Y = rect.Top + (rect.Height - size.Height) / 2;
                using (Brush bru = new SolidBrush(clr))
                {
                    g.DrawString(node.InnerText, font, bru, pt);
                }
                pt.X += size.Width;
            }
        }

        Font get_node_font(DOMNode node, DataGridViewCellStyle cell_style)
        {
            Font font = cell_style.Font;
            if (node.Name == "font" && node.Attributes.ContainsKey("name") && node.Attributes.ContainsKey("size"))
            {
                float font_size = cell_style.Font.Size;
                float.TryParse(node.Attributes["size"], out font_size);
                font = new Font(node.Attributes["name"], font_size);
            }
            return font;
        }

        Color get_node_color(DOMNode node, bool selected, DataGridViewCellStyle cell_style)
        {
            Color clr = selected ? cell_style.SelectionForeColor : cell_style.ForeColor;
            if (node.Name == "font" && node.Attributes.ContainsKey("color"))
            {
                clr = ColorTranslator.FromHtml(node.Attributes["color"]);
                if (selected)
                    clr = Color.FromArgb(~(clr.ToArgb()&0x00FFFFFF));
            }
            return clr;
        }
    }
}
